:- module(regex, [
			compile_pattern/3,
			re_comp/2,
			re_exec/3,
			re_free_pattern/1,
			re_free_registers/1,
			re_init/1,
			re_initialize_registers/1,
			re_match/6,
			re_register/4,
			re_search/5,
			re_search/7,
			re_trail/1]).

:- use_module(chars, [eager_spaces/2]).

re_init(Address) :-
	re_init(case_insensitive, Address).

re_init(case_sensitive, Address) :-
	re_initialize_pattern(0, Address).
re_init(case_insensitive, Address) :-
	re_initialize_pattern(1, Address).


re_comp(PatternChars, PatternBuffer) :-
	re_clean_pattern(PatternChars, CleanPattern),
	length(CleanPattern, PatternLen),
	atom_chars(Pattern, CleanPattern),
	re_compile_pattern(Pattern, PatternLen, PatternBuffer, 0).


re_exec(PatternBuffer, SourceBuff, SourceLen) :-
	re_match(PatternBuffer, SourceBuff, SourceLen, 0, 0, X), X > 0.

%re_exec(PatternBuffer, SourceBuff, SourceLen, Start, End) :-
%	re_initialize_registers(Regs),
%	re_match(PatternBuffer, SourceBuff, SourceLen, 0, Regs, X), X > 0,
%	re_register(Regs, 1, Start, End).

re_search(GrepBuff, ZoneBuff, ZoneSize, ZoneSize, Regs) :-
	re_search(GrepBuff, ZoneBuff, ZoneSize, 0, ZoneSize, Regs).

re_search(GrepBuff, ZoneBuff, ZoneSize, Offset, ZoneSize, Regs) :-
	re_search(GrepBuff, ZoneBuff, ZoneSize, Offset, ZoneSize, Regs, Index),
	Index > -1,
	( true ; NewOffset is Index+1,
                 re_search(GrepBuff, ZoneBuff, ZoneSize, NewOffset, ZoneSize, Regs)
	).		    

/* We treat newlines followed by whitespace as layout charaters rather than pattern characters.
   These layout characters must be stripped before compiling the regular expression.
*/
re_clean_pattern([], []).
re_clean_pattern([10|SpacePattern], CleanPattern) :- !,
	eager_spaces(SpacePattern, Pattern),
	re_clean_pattern(Pattern, CleanPattern).
re_clean_pattern([C|Pattern], [C|CleanPattern]) :-
	re_clean_pattern(Pattern, CleanPattern).

%re_registers_matched(Register, Matched) :-
%	re_num_registers(Register, Regs),
%	re_registers_matched1(Register, 0, Regs, Matched).

%re_registers_matched1(_Register, Num, Num, []) :- !.
%re_registers_matched1(Register, Reg, Regs, Matches) :-
%	% Reg < Regs,
%	re_register(Register, Reg, Start, End),
%	( Start =:= -1 -> Matches = []
%	; End-Start =:= 0 ->
%	  Reg1 is Reg+1,
%	  re_registers_matched1(Register, Reg1, Regs, Matches)
%	; Matches = [Reg|Rest],
%	  Reg1 is Reg+1,
%	  re_registers_matched1(Register, Reg1, Regs, Rest)
%	).

/* re_search(PatternBuff, Str, StrSz, Start, Range, Regs, Results) */

compile_pattern(Pattern, GrepBuff, Regs) :-
	re_init(GrepBuff),
	re_initialize_registers(Regs),

	re_comp(Pattern, GrepBuff).

foreign(re_initialize_pattern, c, re_initialize_pattern(+integer, [-address])).
foreign(re_free_pattern, c, re_free_pattern(+address)).
foreign(re_free_registers, c, re_free_registers(+address)).
foreign(re_initialize_registers, c, re_initialize_registers([-address])).
foreign(re_compile_pattern, c, re_compile_pattern(+string, +integer, +address, [-address])).
foreign(re_match, c, re_match(+address, +address, +integer, +integer, +integer, [-integer])).
foreign(re_search, c, re_search(+address, +address, +integer, +integer, +integer, +address, [-integer])).
%foreign(re_num_registers, c, re_num_registers(+address, [-integer])).
foreign(re_register, c, re_register(+address, +integer, -integer, -integer)).
foreign(re_trail, c, re_trail([-address])).

foreign_file('regex_pl.obj', [re_initialize_pattern, re_free_pattern, re_free_registers, 
%			      re_num_registers, re_register]).
			      re_register,re_trail]).
foreign_file('regex.obj', [re_compile_pattern, re_match, re_search, re_initialize_registers]).

:- load_foreign_files(['regex_pl.obj','regex.obj'], []).
